/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package db4ounit.extensions.fixtures;

import com.db4o.config.*;
import com.db4o.defragment.*;

import db4ounit.extensions.*;

public abstract class AbstractDb4oFixture implements Db4oFixture {

	private final ConfigurationSource _configSource;
	private Configuration _config;

	protected AbstractDb4oFixture(ConfigurationSource configSource) {
		_configSource=configSource;
	}
	
	public void reopen() throws Exception {
		close();
		open();
	}

	public Configuration config() {
		if(_config==null) {
			_config=_configSource.config();
		}
		return _config;
	}
	
	public void clean() {
		doClean();
		resetConfig();
	}
	
	public boolean accept(Class clazz) {
		return true;
	}

	protected abstract void doClean();	
	
	protected void resetConfig() {
		_config=null;
	}
	
	protected void defragment(String fileName) throws Exception{
        String targetFile = fileName + ".defrag.backup";
        DefragmentConfig defragConfig = new DefragmentConfig(fileName, targetFile);
        defragConfig.forceBackupDelete(true);
        defragConfig.db4oConfig(config());
		com.db4o.defragment.Defragment.defrag(defragConfig);
	}
}
